@@ -8,14 +8,13 @@ on the same wire
 from numpy.fft import fft
 import numpy as np
 from . import read_meter
+import pandas as pd
 
 # detect if the calibrator is present
-MIN_CAL_SIGNAL = 5e3  # if |S|<N, detection error
+MIN_CAL_SIGNAL = 10  # if |S|<N, detection error
 # detect if calibrator is on the same phase as before
 # minimum difference between calibration unit vectors
 MIN_CAL_DIFFERENCE = 0.1
-# compensate for FFT details so we get watts
-CAL_SCALE = 246
 # location to store calibration artifacts
 CALIB_DATA = "/opt/configs/cal_data"
 
@@ -82,13 +81,37 @@ def run(cur_phase, meter, replay=False, debug=False):
         data = open(CALIB_DATA + "/%s_ph%d.csv" % (meter_name, cur_phase + 1), 'r')
         prep_buffer = np.loadtxt(data, delimiter=",")
 
-    for i in range(num_sensors):
-        p_coeff = detect_calibrator(prep_buffer[:, 2 * i])
-        q_coeff = detect_calibrator(prep_buffer[:, 2 * i + 1])
+    Fs = 60
+    cal_on =  calibration_config['on_duration']         # calibration load ON time in seconds
+    cal_off = calibration_config['off_duration']        # calibration load OFF time in seconds
+    cal_T = cal_on +  cal_off                           # calibration load period in seconds
+    period = Fs*cal_T                                   # calibration load period in samples
+    f_fund = 1/cal_T                                    # fundamental frequency
 
-        sensor_coeffs.append([p_coeff, q_coeff])
+    # frequency of calibration load
+    if cal_on <=1:
+        k = int(1)
+    else:
+        k = int(cal_on)
+
+    d = cal_on/cal_T                                    # duty cycle 
+    ratio = cal_on/cal_off
+
+
+    Bsize = int(period*3)                               # block size (DFT length)
+    delta_f = Fs/Bsize                                  # frequency spacing
+
+    index_1 = int(k*f_fund*Bsize/Fs)                        # index of harmonic of interest
+
+    sensor_coeffs = []
+    for i in range(num_sensors):
+        p_coeff = detect_calibrator(prep_buffer[:,2*i],period,index_1,Bsize,d,k,ratio)
+        q_coeff = detect_calibrator(prep_buffer[:,2*i+1],period,index_1,Bsize,d,k,ratio)
+       
+        sensor_coeffs.append([p_coeff,q_coeff])
 
     norm_coeffs = np.array(sensor_coeffs)
+
     # ----DEBUG vector visualization----
     if (debug):
         try:
@@ -186,7 +209,7 @@ def run(cur_phase, meter, replay=False, debug=False):
     cal_watts = calibration_config["watts"]
     rms_voltage = float(meter["sensors"]["voltage"]["nominal_rms_voltage"])
     scale_factor = cal_watts * np.sqrt(2) / (rms_voltage)
-    sensor_coeffs = np.array(final_coeffs) / CAL_SCALE / scale_factor
+    sensor_coeffs = np.array(final_coeffs) / scale_factor
 
     # see if we found the calibrator
     if (np.linalg.norm(final_coeffs) < MIN_CAL_SIGNAL):
@@ -242,8 +265,11 @@ def run(cur_phase, meter, replay=False, debug=False):
         x = input("Press [enter] to continue...")
     return (0, "OK")
 
+def rollingmedian(data,window_size):
+    return pd.Series(data).rolling(window = window_size,center=True,min_periods=1).median().values 
 
-def detect_calibrator(x_input):
+
+def detect_calibrator(x_input, PERIOD, XK1_INDEX, Bsize, d, k, ratio):
     """
     For input signal x, find the 3 Hz 
     calibration signal in the waveform and return
@@ -251,10 +277,8 @@ def detect_calibrator(x_input):
     content besides the calibration signal
 
     Calculates the Discrete Fourier Series Coeff for the
-    waveform at 1/3Hz which is the fundamental of the 
-    square wave signal. Multiplying by pi/4 recovers
-    the amplitude of the square wave itself. To increase
-    the robustness of the calculation, the input waveform
+    waveform at the harmonic of interest of the square wave signal.
+    To increase the robustness of the calculation, the input waveform
     is divided in to BSize sample chunks and the DFS coeff's 
     are averaged together
 
@@ -262,30 +286,66 @@ def detect_calibrator(x_input):
     and checking the second harmonic. If it is close to pi then
     this is a positive term, if its close to zero then it is negative
     """
-    PERIOD = 180
-    HALF_PERIOD = 90
-    XK6_INDEX = 6  # depends on nfft (Bsize)
-    XK3_INDEX = 3
-    Bsize = PERIOD * 3  # N periods per block
-    a = np.shape(x_input)
-    # divide the input into blocks
-    Nb = int(np.floor(a[0] / Bsize))  # number of blocks
+    HALF_PERIOD = PERIOD/2
+    XK2_INDEX = 2*XK1_INDEX 
+    a=np.shape(x_input)
+    #divide the input into blocks
+    Nb=int(np.floor(a[0]/Bsize)) #number of blocks
+
+    def is_odd(num):
+        return num % 2 != 0
+
+    on_samples = d*PERIOD
+
+    window = (on_samples/k)
+    if k == 1 and ratio > 1:
+        window = window/ratio
+    window = round(window)
+
+    if (is_odd(window) == False):
+        window += 1
 
     xk_sum = 0
-    for j in range(Nb):
-        N = Bsize
-        x = x_input[(j) * Bsize:(j + 1) * Bsize]
-        X = fft(x)
-        XK3 = X[XK3_INDEX]
-        XK6 = X[XK6_INDEX]
-        xk3 = np.abs(XK3)
-        # correct the time shift
-        M = PERIOD * np.angle(XK3) / (2 * np.pi)
-        m_delta = HALF_PERIOD - M  # set time shift to HALF PERIOD
-        xk6_ph = np.angle(XK6 * np.exp(1j * 2 * np.pi * XK6_INDEX * m_delta / Bsize))
-        if np.abs(xk6_ph) > np.pi / 2:  # cal load is mostly *off*
-            xk3 = -xk3
-        xk_sum = xk_sum + xk3
-    xk_avg = xk_sum / Nb
-    s = xk_avg * np.pi / 4 * 2
-    return s
+    for n in range(Nb):
+        N=Bsize
+        x=x_input[(n)*Bsize:(n+1)*Bsize]
+
+        x = rollingmedian(x,window)
+        Y = fft(x)
+        P1 = (2/Bsize)*(Y[0:(Bsize//2)])
+
+        XK1 = P1[XK1_INDEX]
+        XK2 = P1[XK2_INDEX]
+
+        xk1 = np.abs(XK1)
+        
+        #correct the time shift
+        R=Bsize*np.angle(XK1)/(2*np.pi*XK1_INDEX)
+        xk2_ph=np.angle(XK2*np.exp(-1j*2*np.pi*XK2_INDEX*R/Bsize))
+
+        if(np.abs(xk2_ph)>np.pi/2): 
+            xk1 = -xk1
+
+        if k > 1:
+            if (is_odd(k) == False):
+                xk1 = -xk1
+        else:
+            if ratio > 1:
+                if (is_odd(round(ratio)) == False):
+                    xk1 = -xk1
+
+        xk_sum=xk_sum+xk1
+
+    xk_avg = xk_sum/Nb
+
+    ####################################
+    # a_k = (2*A/k*pi)*sin(k*pi*Tp/T)  #
+    # where Tp is the lengt of pulse   #
+    # and T is the period.             #
+    # The duty cycle, d = Tp/T         #
+    ####################################
+
+    A_scale = k*(np.pi/2) * (1/np.sin(k*np.pi*d))
+    A = xk_avg*A_scale
+
+    return A
