@@ -141,6 +141,44 @@ def peak_detect(data, delta=0.1):
                 lookformax = True
     return peaks
 
+def goertzel(x,f_samp,f_vector):
+    N = len(x)
+    delta_f = f_samp/N
+    ind_vector = f_vector/delta_f
+
+    DFT = np.zeros(len(ind_vector),dtype = np.complex)
+
+    for ind, k in enumerate(ind_vector):
+
+        # precalculation of constants 
+        A = 2*np.pi*k/N
+        B = 2*np.cos(A)
+        C = np.exp(-1j*A)
+        D = np.exp(-1j*A*(N-1))
+
+        # State variables
+        s0 = 0
+        s1 = 0
+        s2 = 0
+
+        # Main loop
+        # number of iterations is one less than the length of the signal
+        for n in range(0, N-1):
+            # new state
+            s0 = x[n] + B*s1 -s2
+            # shifting the state variables
+            s2 = s1
+            s1 = s0
+
+        # Finalizing calculations
+        s0 = x[-1] + B*s1 - s2
+        y = s0 - s1*C               # resulting complex coefficient
+        y = y*D                     # constant substituting the iteration N-1, and correcting the phase at the same time
+     
+        DFT[ind] = 2.0*y/N
+
+    return DFT
+
 
 class FilterError(Exception):
     pass
